/*
Copyright © 2021 NAME HERE <EMAIL ADDRESS>

*/
package cmd

import (
	"fmt"
	"strconv"

	"github.com/spf13/cobra"
	"os"
)

// deleteCmd represents the delete command
var deleteCmd = &cobra.Command{
	Use:   "delete [Id]",
	Args:  cobra.ExactArgs(1),
	Short: "Delete the given entry. Will ask for confirmation",
	Long: `We do not recommend using delete, unless something was truly added by mistake.
	Delete is a hard delete. There is no recovery.`,
	PreRunE: func(cmd *cobra.Command, args []string) error {
		id, err := strconv.Atoi(args[0])
		if err != nil {
			return err
		}
		entry, err := App.GetDiaryEntry(int64(id))
		notes := App.GetDiaryEntryNotes(int64(id))
		if err != nil {
			return err
		}
		fmt.Println("Are you sure you want to delete the following entry? (y/n)")
		PrintEntry(entry)
		PrintNotes(notes)
		var response string
		fmt.Scanln(&response)
		if response == "y" {
			return nil
		} else {
			fmt.Println("Will not delete entry " + args[0])
			os.Exit(0)
		}
		return nil
	},
	Run: func(cmd *cobra.Command, args []string) {
		id, err := strconv.Atoi(args[0])
		if err != nil {
			panic(err)
		}
		_, err = App.DeleteDiaryEntry(int64(id))
		_, err = App.DeleteNotesForDiaryEntry(int64(id))
		if err != nil {
			panic(err)
		}
	},
}

func init() {
	rootCmd.AddCommand(deleteCmd)

	// Here you will define your flags and configuration settings.

	// Cobra supports Persistent Flags which will work for this command
	// and all subcommands, e.g.:
	// deleteCmd.PersistentFlags().String("foo", "", "A help for foo")

	// Cobra supports local flags which will only run when this command
	// is called directly, e.g.:
	// deleteCmd.Flags().BoolP("toggle", "t", false, "Help message for toggle")
}
